﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace AirLib
{
    /// <summary>
    /// PList Parser
    /// <remarks>
    /// Found at http://www.codeproject.com/Tips/406235/A-Simple-PList-Parser-in-Csharp
    /// Coded by Tony Wang
    /// Released under the The Code Project Open License (CPOL)
    /// http://www.codeproject.com/info/cpol10.aspx
    /// </remarks>
    /// </summary>
    internal class PList : Dictionary<string, dynamic>
    {
        public PList()
        {
        }

        public PList(string pListContent)
        {
            Load(pListContent);
        }

        public void Load(string pListContent)
        {
            Clear();

            XDocument doc = XDocument.Parse(pListContent);
            XElement plist = doc.Element("plist");
            XElement dict = plist.Element("dict");

            var dictElements = dict.Elements();
            Parse(this, dictElements);
        }

        private void Parse(PList dict, IEnumerable<XElement> elements)
        {
            for (int i = 0; i < elements.Count(); i += 2)
            {
                XElement key = elements.ElementAt(i);
                XElement val = elements.ElementAt(i + 1);

                dict[key.Value] = ParseValue(val);
            }
        }

        private List<dynamic> ParseArray(IEnumerable<XElement> elements)
        {
            List<dynamic> list = new List<dynamic>();
            foreach (XElement e in elements)
            {
                dynamic one = ParseValue(e);
                list.Add(one);
            }

            return list;
        }

        private dynamic ParseValue(XElement val)
        {
            switch (val.Name.ToString())
            {
                case "string":
                    return val.Value;
                case "integer":
                    return int.Parse(val.Value);
                case "real":
                    return float.Parse(val.Value);
                case "true":
                    return true;
                case "false":
                    return false;
                case "dict":
                    PList plist = new PList();
                    Parse(plist, val.Elements());
                    return plist;
                case "array":
                    List<dynamic> list = ParseArray(val.Elements());
                    return list;
                default:
                    throw new ArgumentException("Unsupported");
            }
        }
    }
}
